@@ -16,13 +16,15 @@ export default {
 		completed: __('Revisit', 'extendify-local'),
 	},
 	type: 'internalLink',
-	dependencies: { plugins: ['all-in-one-seo-pack'] },
-	show: ({ plugins, activePlugins }) => {
+	dependencies: { goals: [], plugins: ['all-in-one-seo-pack'] },
+	show: ({ plugins, goals, activePlugins, userGoals }) => {
 		// They need either extendable or launch completed
 		if (themeSlug !== 'extendable' && !launchCompleted) return false;
-		if (!plugins.length) return true;
+		if (!plugins.length && !goals.length) return true;
 
-		return activePlugins.some((item) => plugins.includes(item));
+		return activePlugins
+			.concat(userGoals)
+			.some((item) => plugins.concat(goals).includes(item));
 	},
 	backgroundImage:
 		'https://images.extendify-cdn.com/assist-tasks/bg-for-seo.webp',
