@@ -4,20 +4,26 @@
  *
  * Functions for formatting data.
  *
- * @package WooCommerce/Functions
+ * @package WooCommerce\Functions
  * @version 2.1.0
  */
 
+use Automattic\WooCommerce\Utilities\I18nUtil;
+use Automattic\WooCommerce\Utilities\NumberUtil;
+
 defined( 'ABSPATH' ) || exit;
 
+// Once WooCommerce requires PHP 7.4, the "$x = $x ?? ''" constructs can be replaced with "$x ??= ''".
+
 /**
  * Converts a string (e.g. 'yes' or 'no') to a bool.
  *
  * @since 3.0.0
- * @param string $string String to convert.
+ * @param string|bool $string String to convert. If a bool is passed it will be returned as-is.
  * @return bool
  */
 function wc_string_to_bool( $string ) {
+	$string = $string ?? '';
 	return is_bool( $string ) ? $string : ( 'yes' === strtolower( $string ) || 1 === $string || 'true' === strtolower( $string ) || '1' === $string );
 }
 
@@ -25,7 +31,7 @@ function wc_string_to_bool( $string ) {
  * Converts a bool to a 'yes' or 'no'.
  *
  * @since 3.0.0
- * @param bool $bool String to convert.
+ * @param bool|string $bool Bool to convert. If a string is passed it will first be converted to a bool.
  * @return string
  */
 function wc_bool_to_string( $bool ) {
@@ -44,6 +50,7 @@ function wc_bool_to_string( $bool ) {
  * @return array
  */
 function wc_string_to_array( $string, $delimiter = ',' ) {
+	$string = $string ?? '';
 	return is_array( $string ) ? $string : array_filter( explode( $delimiter, $string ) );
 }
 
@@ -55,7 +62,7 @@ function wc_string_to_array( $string, $delimiter = ',' ) {
  * @return string
  */
 function wc_sanitize_taxonomy_name( $taxonomy ) {
-	return apply_filters( 'sanitize_taxonomy_name', urldecode( sanitize_title( urldecode( $taxonomy ) ) ), $taxonomy );
+	return apply_filters( 'sanitize_taxonomy_name', urldecode( sanitize_title( urldecode( $taxonomy ?? '' ) ) ), $taxonomy );
 }
 
 /**
@@ -70,7 +77,7 @@ function wc_sanitize_taxonomy_name( $taxonomy ) {
 function wc_sanitize_permalink( $value ) {
 	global $wpdb;
 
-	$value = $wpdb->strip_invalid_text_for_column( $wpdb->options, 'option_value', $value );
+	$value = $wpdb->strip_invalid_text_for_column( $wpdb->options, 'option_value', $value ?? '' );
 
 	if ( is_wp_error( $value ) ) {
 		$value = '';
@@ -103,10 +110,10 @@ function wc_get_filename_from_url( $file_url ) {
  *
  * @param int|float $dimension    Dimension.
  * @param string    $to_unit      Unit to convert to.
- *                                Options: 'in', 'm', 'cm', 'm'.
+ *                                Options: 'in', 'mm', 'cm', 'm'.
  * @param string    $from_unit    Unit to convert from.
  *                                Defaults to ''.
- *                                Options: 'in', 'm', 'cm', 'm'.
+ *                                Options: 'in', 'mm', 'cm', 'm'.
  * @return float
  */
 function wc_get_dimension( $dimension, $to_unit, $from_unit = '' ) {
@@ -214,7 +221,7 @@ function wc_get_weight( $weight, $to_unit, $from_unit = '' ) {
  * @return string
  */
 function wc_trim_zeros( $price ) {
-	return preg_replace( '/' . preg_quote( wc_get_price_decimal_separator(), '/' ) . '0++$/', '', $price );
+	return preg_replace( '/' . preg_quote( wc_get_price_decimal_separator(), '/' ) . '0++$/', '', $price ?? '' );
 }
 
 /**
@@ -225,15 +232,8 @@ function wc_trim_zeros( $price ) {
  * @return float
  */
 function wc_round_tax_total( $value, $precision = null ) {
-	$precision = is_null( $precision ) ? wc_get_price_decimals() : intval( $precision );
-
-	if ( version_compare( PHP_VERSION, '5.3.0', '>=' ) ) {
-		$rounded_tax = round( $value, $precision, wc_get_tax_rounding_mode() ); // phpcs:ignore PHPCompatibility.FunctionUse.NewFunctionParameters.round_modeFound
-	} elseif ( 2 === wc_get_tax_rounding_mode() ) {
-		$rounded_tax = wc_legacy_round_half_down( $value, $precision );
-	} else {
-		$rounded_tax = round( $value, $precision );
-	}
+	$precision   = is_null( $precision ) ? wc_get_price_decimals() : intval( $precision );
+	$rounded_tax = NumberUtil::round( $value, $precision, wc_get_tax_rounding_mode() ); // phpcs:ignore PHPCompatibility.FunctionUse.NewFunctionParameters.round_modeFound
 
 	return apply_filters( 'wc_round_tax_total', $rounded_tax, $value, $precision, WC_TAX_ROUNDING_MODE );
 }
@@ -247,7 +247,7 @@ function wc_round_tax_total( $value, $precision = null ) {
  * @return float
  */
 function wc_legacy_round_half_down( $value, $precision ) {
-	$value = wc_float_to_string( $value );
+	$value = wc_float_to_string( $value ) ?? '';
 
 	if ( false !== strstr( $value, '.' ) ) {
 		$value = explode( '.', $value );
@@ -259,7 +259,7 @@ function wc_legacy_round_half_down( $value, $precision ) {
 		$value = implode( '.', $value );
 	}
 
-	return round( floatval( $value ), $precision );
+	return NumberUtil::round( floatval( $value ), $precision );
 }
 
 /**
@@ -276,7 +276,7 @@ function wc_format_refund_total( $amount ) {
 /**
  * Format decimal numbers ready for DB storage.
  *
- * Sanitize, remove decimals, and optionally round + trim off zeros.
+ * Sanitize, optionally remove decimals, and optionally round + trim off zeros.
  *
  * This function does not remove thousands - this should be done before passing a value to the function.
  *
@@ -286,6 +286,8 @@ function wc_format_refund_total( $amount ) {
  * @return string
  */
 function wc_format_decimal( $number, $dp = false, $trim_zeros = false ) {
+	$number = $number ?? '';
+
 	$locale   = localeconv();
 	$decimals = array( wc_get_price_decimal_separator(), $locale['decimal_point'], $locale['mon_decimal_point'] );
 
@@ -343,14 +345,16 @@ function wc_format_localized_price( $value ) {
 }
 
 /**
- * Format a decimal with PHP Locale settings.
+ * Format a decimal with the decimal separator for prices or PHP Locale settings.
  *
  * @param  string $value Decimal to localize.
  * @return string
  */
 function wc_format_localized_decimal( $value ) {
-	$locale = localeconv();
-	return apply_filters( 'woocommerce_format_localized_decimal', str_replace( '.', $locale['decimal_point'], strval( $value ) ), $value );
+	$locale        = localeconv();
+	$decimal_point = isset( $locale['decimal_point'] ) ? $locale['decimal_point'] : '.';
+	$decimal       = ( ! empty( wc_get_price_decimal_separator() ) ) ? wc_get_price_decimal_separator() : $decimal_point;
+	return apply_filters( 'woocommerce_format_localized_decimal', str_replace( '.', $decimal, strval( $value ) ), $value );
 }
 
 /**
@@ -367,15 +371,24 @@ function wc_format_coupon_code( $value ) {
 /**
  * Sanitize a coupon code.
  *
- * Uses sanitize_post_field since coupon codes are stored as
- * post_titles - the sanitization and escaping must match.
+ * Uses sanitize_post_field since coupon codes are stored as post_titles - the sanitization and escaping must match.
+ *
+ * Due to the unfiltered_html captability that some (admin) users have, we need to account for slashes.
+ *
+ * The html_entity_decode() call handles coupon codes that contain special characters like ampersands (&), quotes ("),
+ * and other HTML entities. Without this decoding step, coupon codes with special characters would fail to match
+ * during application, causing legitimate coupons to be rejected.
+ *
+ * @see WC_Cart_Test::test_coupon_codes_with_special_characters
  *
  * @since  3.6.0
+ * @since  10.0.0 Decode HTML entities here instead of via woocommerce_coupon_code filter.
  * @param  string $value Coupon code to format.
  * @return string
  */
 function wc_sanitize_coupon_code( $value ) {
-	return sanitize_post_field( 'post_title', $value, 0, 'db' );
+	$value = wp_kses( sanitize_post_field( 'post_title', html_entity_decode( $value ?? '', ENT_COMPAT, get_bloginfo( 'charset' ) ), 0, 'db' ), 'entities' );
+	return current_user_can( 'unfiltered_html' ) ? $value : stripslashes( $value );
 }
 
 /**
@@ -415,7 +428,7 @@ function wc_check_invalid_utf8( $var ) {
  * @return string
  */
 function wc_sanitize_textarea( $var ) {
-	return implode( "\n", array_map( 'wc_clean', explode( "\n", $var ) ) );
+	return implode( "\n", array_map( 'wc_clean', explode( "\n", $var ?? '' ) ) );
 }
 
 /**
@@ -428,7 +441,7 @@ function wc_sanitize_textarea( $var ) {
 function wc_sanitize_tooltip( $var ) {
 	return htmlspecialchars(
 		wp_kses(
-			html_entity_decode( $var ),
+			html_entity_decode( $var ?? '' ),
 			array(
 				'br'     => array(),
 				'em'     => array(),
@@ -553,6 +566,8 @@ function wc_get_price_decimals() {
  *                                      Defaults the result of wc_get_price_decimals().
  *     @type string $price_format       Price format depending on the currency position.
  *                                      Defaults the result of get_woocommerce_price_format().
+ *     @type bool   $in_span            Whether to enclose the formatted price in an HTML <span> element.
+ *                                      Defaults to true.
  * }
  * @return string
  */
@@ -568,21 +583,52 @@ function wc_price( $price, $args = array() ) {
 				'thousand_separator' => wc_get_price_thousand_separator(),
 				'decimals'           => wc_get_price_decimals(),
 				'price_format'       => get_woocommerce_price_format(),
+				'in_span'            => true,
+				'aria-hidden'        => false,
 			)
 		)
 	);
 
+	$original_price = $price;
+
+	// Convert to float to avoid issues on PHP 8.
+	$price = (float) $price;
+
 	$unformatted_price = $price;
 	$negative          = $price < 0;
-	$price             = apply_filters( 'raw_woocommerce_price', floatval( $negative ? $price * -1 : $price ) );
-	$price             = apply_filters( 'formatted_woocommerce_price', number_format( $price, $args['decimals'], $args['decimal_separator'], $args['thousand_separator'] ), $price, $args['decimals'], $args['decimal_separator'], $args['thousand_separator'] );
+
+	/**
+	 * Filter raw price.
+	 *
+	 * @param float        $raw_price      Raw price.
+	 * @param float|string $original_price Original price as float, or empty string. Since 5.0.0.
+	 */
+	$price = apply_filters( 'raw_woocommerce_price', $negative ? $price * -1 : $price, $original_price );
+
+	/**
+	 * Filter formatted price.
+	 *
+	 * @param float        $formatted_price    Formatted price.
+	 * @param float        $price              Unformatted price.
+	 * @param int          $decimals           Number of decimals.
+	 * @param string       $decimal_separator  Decimal separator.
+	 * @param string       $thousand_separator Thousand separator.
+	 * @param float|string $original_price     Original price as float, or empty string. Since 5.0.0.
+	 */
+	$price = apply_filters( 'formatted_woocommerce_price', number_format( $price, $args['decimals'], $args['decimal_separator'], $args['thousand_separator'] ), $price, $args['decimals'], $args['decimal_separator'], $args['thousand_separator'], $original_price );
 
 	if ( apply_filters( 'woocommerce_price_trim_zeros', false ) && $args['decimals'] > 0 ) {
 		$price = wc_trim_zeros( $price );
 	}
 
-	$formatted_price = ( $negative ? '-' : '' ) . sprintf( $args['price_format'], '<span class="woocommerce-Price-currencySymbol">' . get_woocommerce_currency_symbol( $args['currency'] ) . '</span>', $price );
-	$return          = '<span class="woocommerce-Price-amount amount">' . $formatted_price . '</span>';
+	if ( $args['in_span'] ) {
+		$formatted_price = ( $negative ? '-' : '' ) . sprintf( $args['price_format'], '<span class="woocommerce-Price-currencySymbol">' . get_woocommerce_currency_symbol( $args['currency'] ) . '</span>', $price );
+		$aria_hidden     = $args['aria-hidden'] ? ' aria-hidden="true"' : '';
+		$return          = '<span class="woocommerce-Price-amount amount"' . $aria_hidden . '><bdi>' . $formatted_price . '</bdi></span>';
+	} else {
+		$formatted_price = ( $negative ? '-' : '' ) . sprintf( $args['price_format'], get_woocommerce_currency_symbol( $args['currency'] ), $price );
+		$return          = $formatted_price;
+	}
 
 	if ( $args['ex_tax_label'] && wc_tax_enabled() ) {
 		$return .= ' <small class="woocommerce-Price-taxLabel tax_label">' . WC()->countries->ex_tax_or_vat() . '</small>';
@@ -591,12 +637,13 @@ function wc_price( $price, $args = array() ) {
 	/**
 	 * Filters the string of price markup.
 	 *
-	 * @param string $return            Price HTML markup.
-	 * @param string $price             Formatted price.
-	 * @param array  $args              Pass on the args.
-	 * @param float  $unformatted_price Price as float to allow plugins custom formatting. Since 3.2.0.
+	 * @param string       $return            Price HTML markup.
+	 * @param string       $price             Formatted price.
+	 * @param array        $args              Pass on the args.
+	 * @param float        $unformatted_price Price as float to allow plugins custom formatting. Since 3.2.0.
+	 * @param float|string $original_price    Original price as float, or empty string. Since 5.0.0.
 	 */
-	return apply_filters( 'wc_price', $return, $price, $args, $unformatted_price );
+	return apply_filters( 'wc_price', $return, $price, $args, $unformatted_price, $original_price );
 }
 
 /**
@@ -608,6 +655,8 @@ function wc_price( $price, $args = array() ) {
  * @return int
  */
 function wc_let_to_num( $size ) {
+	$size = $size ?? '';
+
 	$l   = substr( $size, -1 );
 	$ret = (int) substr( $size, 0, -1 );
 	switch ( strtoupper( $l ) ) {
@@ -636,7 +685,12 @@ function wc_let_to_num( $size ) {
  * @return string
  */
 function wc_date_format() {
-	return apply_filters( 'woocommerce_date_format', get_option( 'date_format' ) );
+	$date_format = get_option( 'date_format' );
+	if ( empty( $date_format ) ) {
+		// Return default date format if the option is empty.
+		$date_format = 'F j, Y';
+	}
+	return apply_filters( 'woocommerce_date_format', $date_format );
 }
 
 /**
@@ -645,7 +699,12 @@ function wc_date_format() {
  * @return string
  */
 function wc_time_format() {
-	return apply_filters( 'woocommerce_time_format', get_option( 'time_format' ) );
+	$time_format = get_option( 'time_format' );
+	if ( empty( $time_format ) ) {
+		// Return default time format if the option is empty.
+		$time_format = 'g:i a';
+	}
+	return apply_filters( 'woocommerce_time_format', $time_format );
 }
 
 /**
@@ -659,6 +718,8 @@ function wc_time_format() {
  * @return int
  */
 function wc_string_to_timestamp( $time_string, $from_timestamp = null ) {
+	$time_string = $time_string ?? '';
+
 	$original_timezone = date_default_timezone_get();
 
 	// @codingStandardsIgnoreStart
@@ -684,6 +745,8 @@ function wc_string_to_timestamp( $time_string, $from_timestamp = null ) {
  * @return WC_DateTime
  */
 function wc_string_to_datetime( $time_string ) {
+	$time_string = $time_string ?? '';
+
 	// Strings are defined in local WP timezone. Convert to UTC.
 	if ( 1 === preg_match( '/^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2})(Z|((-|\+)\d{2}:\d{2}))$/', $time_string, $date_bits ) ) {
 		$offset    = ! empty( $date_bits[7] ) ? iso8601_timezone_to_offset( $date_bits[7] ) : wc_timezone_offset();
@@ -713,6 +776,11 @@ function wc_string_to_datetime( $time_string ) {
  * @return string PHP timezone string for the site
  */
 function wc_timezone_string() {
+	// Added in WordPress 5.3 Ref https://developer.wordpress.org/reference/functions/wp_timezone_string/.
+	if ( function_exists( 'wp_timezone_string' ) ) {
+		return wp_timezone_string();
+	}
+
 	// If site timezone string exists, return it.
 	$timezone = get_option( 'timezone_string' );
 	if ( $timezone ) {
@@ -720,13 +788,13 @@ function wc_timezone_string() {
 	}
 
 	// Get UTC offset, if it isn't set then return UTC.
-	$utc_offset = intval( get_option( 'gmt_offset', 0 ) );
-	if ( 0 === $utc_offset ) {
+	$utc_offset = floatval( get_option( 'gmt_offset', 0 ) );
+	if ( ! is_numeric( $utc_offset ) || 0.0 === $utc_offset ) {
 		return 'UTC';
 	}
 
 	// Adjust UTC offset from hours to seconds.
-	$utc_offset *= 3600;
+	$utc_offset = (int) ( $utc_offset * 3600 );
 
 	// Attempt to guess the timezone string from the UTC offset.
 	$timezone = timezone_name_from_abbr( '', $utc_offset );
@@ -786,7 +854,7 @@ if ( ! function_exists( 'wc_rgb_from_hex' ) ) {
 	 * @return array
 	 */
 	function wc_rgb_from_hex( $color ) {
-		$color = str_replace( '#', '', $color );
+		$color = str_replace( '#', '', $color ?? '000' );
 		// Convert shorthand colors to full format, e.g. "FFF" -> "FFFFFF".
 		$color = preg_replace( '~^(.)(.)(.)$~', '$1$1$2$2$3$3', $color );
 
@@ -815,7 +883,7 @@ if ( ! function_exists( 'wc_hex_darker' ) ) {
 
 		foreach ( $base as $k => $v ) {
 			$amount      = $v / 100;
-			$amount      = round( $amount * $factor );
+			$amount      = NumberUtil::round( $amount * $factor );
 			$new_decimal = $v - $amount;
 
 			$new_hex_component = dechex( $new_decimal );
@@ -846,7 +914,7 @@ if ( ! function_exists( 'wc_hex_lighter' ) ) {
 		foreach ( $base as $k => $v ) {
 			$amount      = 255 - $v;
 			$amount      = $amount / 100;
-			$amount      = round( $amount * $factor );
+			$amount      = NumberUtil::round( $amount * $factor );
 			$new_decimal = $v + $amount;
 
 			$new_hex_component = dechex( $new_decimal );
@@ -869,7 +937,7 @@ if ( ! function_exists( 'wc_hex_is_light' ) ) {
 	 * @return bool  True if a light color.
 	 */
 	function wc_hex_is_light( $color ) {
-		$hex = str_replace( '#', '', $color );
+		$hex = str_replace( '#', '', $color ?? '' );
 
 		$c_r = hexdec( substr( $hex, 0, 2 ) );
 		$c_g = hexdec( substr( $hex, 2, 2 ) );
@@ -907,7 +975,7 @@ if ( ! function_exists( 'wc_format_hex' ) ) {
 	 * @return string|null
 	 */
 	function wc_format_hex( $hex ) {
-		$hex = trim( str_replace( '#', '', $hex ) );
+		$hex = trim( str_replace( '#', '', $hex ?? '' ) );
 
 		if ( strlen( $hex ) === 3 ) {
 			$hex = $hex[0] . $hex[0] . $hex[1] . $hex[1] . $hex[2] . $hex[2];
@@ -925,9 +993,12 @@ if ( ! function_exists( 'wc_format_hex' ) ) {
  * @return string
  */
 function wc_format_postcode( $postcode, $country ) {
-	$postcode = wc_normalize_postcode( $postcode );
+	$postcode = wc_normalize_postcode( $postcode ?? '' );
 
 	switch ( $country ) {
+		case 'SE':
+			$postcode = substr_replace( $postcode, ' ', -2, 0 );
+			break;
 		case 'CA':
 		case 'GB':
 			$postcode = substr_replace( $postcode, ' ', -3, 0 );
@@ -945,12 +1016,25 @@ function wc_format_postcode( $postcode, $country ) {
 		case 'PT':
 			$postcode = substr_replace( $postcode, '-', 4, 0 );
 			break;
+		case 'PR':
 		case 'US':
+		case 'MN':
 			$postcode = rtrim( substr_replace( $postcode, '-', 5, 0 ), '-' );
 			break;
 		case 'NL':
 			$postcode = substr_replace( $postcode, ' ', 4, 0 );
 			break;
+		case 'LV':
+			$postcode = preg_replace( '/^(LV)?-?(\d+)$/', 'LV-${2}', $postcode );
+			break;
+		case 'CZ':
+		case 'SK':
+			$postcode = preg_replace( "/^({$country})-?(\d+)$/", '${1}-${2}', $postcode );
+			$postcode = substr_replace( $postcode, ' ', -2, 0 );
+			break;
+		case 'DK':
+			$postcode = preg_replace( '/^(DK)(.+)$/', '${1}-${2}', $postcode );
+			break;
 	}
 
 	return apply_filters( 'woocommerce_format_postcode', trim( $postcode ), $country );
@@ -966,7 +1050,7 @@ function wc_format_postcode( $postcode, $country ) {
  * @return string
  */
 function wc_normalize_postcode( $postcode ) {
-	return preg_replace( '/[\s\-]/', '', trim( wc_strtoupper( $postcode ) ) );
+	return preg_replace( '/[\s\-]/', '', trim( wc_strtoupper( $postcode ?? '' ) ) );
 }
 
 /**
@@ -976,6 +1060,8 @@ function wc_normalize_postcode( $postcode ) {
  * @return string
  */
 function wc_format_phone_number( $phone ) {
+	$phone = $phone ?? '';
+
 	if ( ! WC_Validation::is_phone( $phone ) ) {
 		return '';
 	}
@@ -991,7 +1077,7 @@ function wc_format_phone_number( $phone ) {
  * @return string
  */
 function wc_sanitize_phone_number( $phone ) {
-	return preg_replace( '/[^\d+]/', '', $phone );
+	return preg_replace( '/[^\d+]/', '', $phone ?? '' );
 }
 
 /**
@@ -1002,6 +1088,7 @@ function wc_sanitize_phone_number( $phone ) {
  * @return string
  */
 function wc_strtoupper( $string ) {
+	$string = $string ?? '';
 	return function_exists( 'mb_strtoupper' ) ? mb_strtoupper( $string ) : strtoupper( $string );
 }
 
@@ -1014,6 +1101,7 @@ function wc_strtoupper( $string ) {
  * @return string
  */
 function wc_strtolower( $string ) {
+	$string = $string ?? '';
 	return function_exists( 'mb_strtolower' ) ? mb_strtolower( $string ) : strtolower( $string );
 }
 
@@ -1028,6 +1116,8 @@ function wc_strtolower( $string ) {
  * @return string
  */
 function wc_trim_string( $string, $chars = 200, $suffix = '...' ) {
+	$string = $string ?? '';
+
 	if ( strlen( $string ) > $chars ) {
 		if ( function_exists( 'mb_substr' ) ) {
 			$string = mb_substr( $string, 0, ( $chars - mb_strlen( $suffix ) ) ) . $suffix;
@@ -1046,6 +1136,7 @@ function wc_trim_string( $string, $chars = 200, $suffix = '...' ) {
  * @return string
  */
 function wc_format_content( $raw_string ) {
+	$raw_string = $raw_string ?? '';
 	return apply_filters( 'woocommerce_format_content', apply_filters( 'woocommerce_short_description', $raw_string ), $raw_string );
 }
 
@@ -1086,7 +1177,7 @@ function wc_format_product_short_description( $content ) {
  * @return string
  */
 function wc_format_option_price_separators( $value, $option, $raw_value ) {
-	return wp_kses_post( $raw_value );
+	return wp_kses_post( $raw_value ?? '' );
 }
 add_filter( 'woocommerce_admin_settings_sanitize_option_woocommerce_price_decimal_sep', 'wc_format_option_price_separators', 10, 3 );
 add_filter( 'woocommerce_admin_settings_sanitize_option_woocommerce_price_thousand_sep', 'wc_format_option_price_separators', 10, 3 );
@@ -1120,7 +1211,8 @@ function wc_format_option_hold_stock_minutes( $value, $option, $raw_value ) {
 	wp_clear_scheduled_hook( 'woocommerce_cancel_unpaid_orders' );
 
 	if ( '' !== $value ) {
-		wp_schedule_single_event( time() + ( absint( $value ) * 60 ), 'woocommerce_cancel_unpaid_orders' );
+		$cancel_unpaid_interval = apply_filters( 'woocommerce_cancel_unpaid_orders_interval_minutes', absint( $value ) );
+		wp_schedule_single_event( time() + ( absint( $cancel_unpaid_interval ) * 60 ), 'woocommerce_cancel_unpaid_orders' );
 	}
 
 	return $value;
@@ -1135,7 +1227,7 @@ add_filter( 'woocommerce_admin_settings_sanitize_option_woocommerce_hold_stock_m
  * @return string
  */
 function wc_sanitize_term_text_based( $term ) {
-	return trim( wp_strip_all_tags( wp_unslash( $term ) ) );
+	return trim( wp_strip_all_tags( wp_unslash( $term ?? '' ) ) );
 }
 
 if ( ! function_exists( 'wc_make_numeric_postcode' ) ) {
@@ -1150,13 +1242,13 @@ if ( ! function_exists( 'wc_make_numeric_postcode' ) ) {
 	 * @return string
 	 */
 	function wc_make_numeric_postcode( $postcode ) {
-		$postcode           = str_replace( array( ' ', '-' ), '', $postcode );
+		$postcode           = str_replace( array( ' ', '-' ), '', $postcode ?? '' );
 		$postcode_length    = strlen( $postcode );
 		$letters_to_numbers = array_merge( array( 0 ), range( 'A', 'Z' ) );
 		$letters_to_numbers = array_flip( $letters_to_numbers );
 		$numeric_postcode   = '';
 
-		for ( $i = 0; $i < $postcode_length; $i ++ ) {
+		for ( $i = 0; $i < $postcode_length; $i++ ) {
 			if ( is_numeric( $postcode[ $i ] ) ) {
 				$numeric_postcode .= str_pad( $postcode[ $i ], 2, '0', STR_PAD_LEFT );
 			} elseif ( isset( $letters_to_numbers[ $postcode[ $i ] ] ) ) {
@@ -1183,7 +1275,7 @@ function wc_format_stock_for_display( $product ) {
 
 	switch ( get_option( 'woocommerce_stock_format' ) ) {
 		case 'low_amount':
-			if ( $stock_amount <= get_option( 'woocommerce_notify_low_stock_amount' ) ) {
+			if ( $stock_amount <= wc_get_low_stock_amount( $product ) ) {
 				/* translators: %s: stock amount */
 				$display = sprintf( __( 'Only %s left in stock', 'woocommerce' ), wc_format_stock_quantity_for_display( $stock_amount, $product ) );
 			}
@@ -1222,7 +1314,28 @@ function wc_format_stock_quantity_for_display( $stock_quantity, $product ) {
  * @return string
  */
 function wc_format_sale_price( $regular_price, $sale_price ) {
-	$price = '<del>' . ( is_numeric( $regular_price ) ? wc_price( $regular_price ) : $regular_price ) . '</del> <ins>' . ( is_numeric( $sale_price ) ? wc_price( $sale_price ) : $sale_price ) . '</ins>';
+	// Format the prices.
+	$formatted_regular_price = is_numeric( $regular_price ) ? wc_price( $regular_price ) : $regular_price;
+	$formatted_sale_price    = is_numeric( $sale_price ) ? wc_price( $sale_price ) : $sale_price;
+
+	// Strikethrough pricing.
+	$price = '<del aria-hidden="true">' . $formatted_regular_price . '</del> ';
+
+	// For accessibility (a11y) we'll also display that information to screen readers.
+	$price .= '<span class="screen-reader-text">';
+	// translators: %s is a product's regular price.
+	$price .= esc_html( sprintf( __( 'Original price was: %s.', 'woocommerce' ), wp_strip_all_tags( $formatted_regular_price ) ) );
+	$price .= '</span>';
+
+	// Add the sale price.
+	$price .= '<ins aria-hidden="true">' . $formatted_sale_price . '</ins>';
+
+	// For accessibility (a11y) we'll also display that information to screen readers.
+	$price .= '<span class="screen-reader-text">';
+	// translators: %s is a product's current (sale) price.
+	$price .= esc_html( sprintf( __( 'Current price is: %s.', 'woocommerce' ), wp_strip_all_tags( $formatted_sale_price ) ) );
+	$price .= '</span>';
+
 	return apply_filters( 'woocommerce_format_sale_price', $price, $regular_price, $sale_price );
 }
 
@@ -1235,7 +1348,15 @@ function wc_format_sale_price( $regular_price, $sale_price ) {
  */
 function wc_format_price_range( $from, $to ) {
 	/* translators: 1: price from 2: price to */
-	$price = sprintf( _x( '%1$s &ndash; %2$s', 'Price range: from-to', 'woocommerce' ), is_numeric( $from ) ? wc_price( $from ) : $from, is_numeric( $to ) ? wc_price( $to ) : $to );
+	$price  = sprintf( _x( '%1$s <span aria-hidden="true">&ndash;</span> %2$s', 'Price range: from-to', 'woocommerce' ), is_numeric( $from ) ? wc_price( $from, array( 'aria-hidden' => true ) ) : $from, is_numeric( $to ) ? wc_price( $to, array( 'aria-hidden' => true ) ) : $to );
+	$price .= '<span class="screen-reader-text">';
+	$price .= sprintf(
+		/* translators: 1: price from 2: price to */
+		__( 'Price range: %1$s through %2$s', 'woocommerce' ),
+		is_numeric( $from ) ? wp_strip_all_tags( wc_price( $from ) ) : wp_strip_all_tags( $from ),
+		is_numeric( $to ) ? wp_strip_all_tags( wc_price( $to ) ) : wp_strip_all_tags( $to )
+	);
+	$price .= '</span>';
 	return apply_filters( 'woocommerce_format_price_range', $price, $from, $to );
 }
 
@@ -1250,7 +1371,14 @@ function wc_format_weight( $weight ) {
 	$weight_string = wc_format_localized_decimal( $weight );
 
 	if ( ! empty( $weight_string ) ) {
-		$weight_string .= ' ' . get_option( 'woocommerce_weight_unit' );
+		$weight_label = I18nUtil::get_weight_unit_label( get_option( 'woocommerce_weight_unit' ) );
+
+		$weight_string = sprintf(
+			// translators: 1. A formatted number; 2. A label for a weight unit of measure. E.g. 2.72 kg.
+			_x( '%1$s %2$s', 'formatted weight', 'woocommerce' ),
+			$weight_string,
+			$weight_label
+		);
 	} else {
 		$weight_string = __( 'N/A', 'woocommerce' );
 	}
@@ -1269,7 +1397,14 @@ function wc_format_dimensions( $dimensions ) {
 	$dimension_string = implode( ' &times; ', array_filter( array_map( 'wc_format_localized_decimal', $dimensions ) ) );
 
 	if ( ! empty( $dimension_string ) ) {
-		$dimension_string .= ' ' . get_option( 'woocommerce_dimension_unit' );
+		$dimension_label = I18nUtil::get_dimensions_unit_label( get_option( 'woocommerce_dimension_unit' ) );
+
+		$dimension_string = sprintf(
+			// translators: 1. A formatted number; 2. A label for a dimensions unit of measure. E.g. 3.14 cm.
+			_x( '%1$s %2$s', 'formatted dimensions', 'woocommerce' ),
+			$dimension_string,
+			$dimension_label
+		);
 	} else {
 		$dimension_string = __( 'N/A', 'woocommerce' );
 	}
@@ -1306,7 +1441,7 @@ function wc_format_datetime( $date, $format = '' ) {
 function wc_do_oembeds( $content ) {
 	global $wp_embed;
 
-	$content = $wp_embed->autoembed( $content );
+	$content = $wp_embed->autoembed( $content ?? '' );
 
 	return $content;
 }
@@ -1456,8 +1591,53 @@ function wc_parse_relative_date_option( $raw_value ) {
  * @return string
  */
 function wc_sanitize_endpoint_slug( $raw_value ) {
-	return sanitize_title( $raw_value );
+	return sanitize_title( $raw_value ?? '' );
 }
+
+/**
+ * Removes useless non-displayable and problematic Unicode characters from a string.
+ *
+ * This function eliminates characters that can cause formatting issues, invisible text,
+ * or unexpected behavior in copy-pasted text. Specifically, it removes:
+ *
+ * - **Soft hyphen (`U+00AD`)** – Invisible unless text is broken across lines.
+ * - **Zero-width spaces & joiners (`U+200B–U+200D`)** – Invisible and can cause copy/paste issues.
+ * - **Directional markers (`U+200E–U+200F`, `U+202A–U+202E`)** – Can affect text rendering.
+ * - **Byte Order Mark (BOM) (`U+FEFF`)** – Can interfere with encoding.
+ * - **Interlinear annotation characters (`U+FFF9–U+FFFB`)** – Rarely used and unnecessary in checkout fields.
+ *
+ * It does **not** remove:
+ *
+ * - **Non-breaking space (`U+00A0`)** – Useful for preventing line breaks in addresses.
+ * - **Word joiner (`U+2060`)** – Sometimes needed for proper text rendering in certain scripts.
+ *
+ * @param string $raw_value The input string to sanitize.
+ *
+ * @return string The sanitized string without problematic characters.
+ * @since 9.9.0
+ */
+function wc_remove_non_displayable_chars( string $raw_value ): string {
+	$remove_chars = array(
+		"\u{00AD}", // Soft Hyphen.
+		"\u{200B}", // Zero Width Space.
+		"\u{200C}", // Zero Width Non-Joiner.
+		"\u{200D}", // Zero Width Joiner.
+		"\u{200E}", // Left-to-Right Mark.
+		"\u{200F}", // Right-to-Left Mark.
+		"\u{202A}", // Left-to-Right Embedding.
+		"\u{202B}", // Right-to-Left Embedding.
+		"\u{202C}", // Pop Directional Formatting.
+		"\u{202D}", // Left-to-Right Override.
+		"\u{202E}", // Right-to-Left Override.
+		"\u{FEFF}", // Byte Order Mark (BOM).
+		"\u{FFF9}", // Interlinear Annotation Anchor.
+		"\u{FFFA}", // Interlinear Annotation Separator.
+		"\u{FFFB}", // Interlinear Annotation Terminator.
+	);
+
+	return str_replace( $remove_chars, '', $raw_value );
+}
+
 add_filter( 'woocommerce_admin_settings_sanitize_option_woocommerce_checkout_pay_endpoint', 'wc_sanitize_endpoint_slug', 10, 1 );
 add_filter( 'woocommerce_admin_settings_sanitize_option_woocommerce_checkout_order_received_endpoint', 'wc_sanitize_endpoint_slug', 10, 1 );
 add_filter( 'woocommerce_admin_settings_sanitize_option_woocommerce_myaccount_add_payment_method_endpoint', 'wc_sanitize_endpoint_slug', 10, 1 );
